@@ -8,6 +8,7 @@
  * SPDX-License-Identifier: EPL-2.0
  **********************************************************************/
 
+import * as fs from 'fs-extra'
 import * as Listr from 'listr'
 import * as path from 'path'
 
@@ -51,7 +52,12 @@ export class CertManagerTasks {
         title: 'Deploy cert-manager',
         enabled: ctx => !ctx.certManagerInstalled,
         task: async (ctx: any, task: any) => {
-          const yamlPath = path.join(flags.templates, '..', 'installers', 'cert-manager.yml')
+          let yamlPath = path.join(flags.templates, 'cert-manager', 'cert-manager.yaml')
+          if (! await fs.pathExists(yamlPath)) {
+            // Older Che versions don't have Cert Manager install yaml in templates
+            // Try to use embedded in crwctl version
+            yamlPath = path.join(__dirname, '../../../installers/cert-manager.yml')
+          }
           // Apply additional --validate=false flag to be able to deploy Cert Manager on Kubernetes v1.15.4 or below
           await this.kubeHelper.applyResource(yamlPath, '--validate=false')
           ctx.certManagerInstalled = true
